from abc import abstractmethod, ABC

import pandas as pd
import pydantic

from pyfabrik.models import FabrikRawReadResponse, FabrikException


class DataFrameFacade(ABC):
    @abstractmethod
    def read_data_frame(self, r: FabrikRawReadResponse) -> pd.DataFrame:
        pass


class DefaultDataFrameFacade(DataFrameFacade):
    class ParquetContext(pydantic.BaseModel):
        landing_zone_kind: str
        path: str

    def read_data_frame(self, r: FabrikRawReadResponse) -> pd.DataFrame:
        if r.format == "PARQUET":
            return DefaultDataFrameFacade.read_parquet(r)

        raise FabrikException(f"Unsupported format `{r.format}`.")

    @staticmethod
    def read_parquet(r: FabrikRawReadResponse) -> pd.DataFrame:
        ctx = DefaultDataFrameFacade.ParquetContext.parse_obj(r.context)
        return pd.read_parquet(ctx.path)



import io
import os
import re

from setuptools import find_packages
from setuptools import setup


def read(filename):
    filename = os.path.join(os.path.dirname(__file__), filename)
    text_type = type(u"")
    with io.open(filename, mode="r", encoding="utf-8") as fd:
        return re.sub(text_type(r":[a-z]+:`~?(.*?)`"), text_type(r"``\1``"), fd.read())


with open("requirements.txt") as f:
    requirements = f.read().splitlines()


setup(
    name="oafabrik",
    version="0.3.0",
    url="https://github.com/openaristos/pyfabrik",
    license="AGPLv3",
    author="Aristos Data, LLC.",
    author_email="hello@openaristos.io",
    description="A Python SDK for Fabrik",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=("tests",)),
    install_requires=requirements,
    keywords=["openaristos", "fabrik"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
        "Topic :: Scientific/Engineering :: Information Analysis",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "Intended Audience :: Financial and Insurance Industry",
        "Intended Audience :: Developers",
        "Intended Audience :: Healthcare Industry",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Telecommunications Industry",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
)



from typing import Any

import pydantic
import pandas as pd

from pydantic import typing


class FabrikException(RuntimeError):
    pass


class FabrikReadRequest(pydantic.BaseModel):
    definition: str
    warehouse: str
    dialect: str = "dotted"
    format: str = "parquet"


class FabrikReadResponse(pydantic.BaseModel):
    class Config:
        arbitrary_types_allowed = True

    df: pd.DataFrame


class FabrikWriteRequest(pydantic.BaseModel):
    class Config:
        arbitrary_types_allowed = True

    df: pd.DataFrame
    path: str
    mode: str = "auto"


class FabrikWriteResponse(pydantic.BaseModel):
    pass


class FabrikRawReadResponse(pydantic.BaseModel):
    format: str
    context: typing.Dict[Any, Any]



import concurrent.futures
import typing
from concurrent.futures._base import Future

import requests

from .data import DataFrameFacade
from .models import (
    FabrikReadRequest,
    FabrikReadResponse,
    FabrikWriteRequest,
    FabrikException,
    FabrikRawReadResponse,
    FabrikWriteResponse,
)


def _perform_read(endpoint: str, req: FabrikReadRequest, session: requests.Session,
                  df_facade: DataFrameFacade) -> FabrikReadResponse:
    try:
        res = session.post(endpoint + "/read", json=req.dict())
        res.raise_for_status()

        fabrik_read_res: FabrikRawReadResponse = FabrikRawReadResponse.parse_obj(
            res.json()
        )

        return FabrikReadResponse(
            df=df_facade.read_data_frame(fabrik_read_res)
        )

    except Exception as ex:
        raise FabrikException("Failed to execute read request.", ex)
    pass


class FabrikClient(object):
    def __init__(self, endpoint: str, token: str, df_facade: DataFrameFacade):
        self.endpoint = endpoint
        self.token = token
        self.df_facade = df_facade

        self.session = requests.session()
        self.session.headers.update({"authorization": "Bearer " + self.token})

    def read_all(self, r: typing.List[FabrikReadRequest]) -> typing.List[FabrikReadResponse]:
        futures: typing.List[Future] = []
        results: typing.List[FabrikReadResponse] = []

        with concurrent.futures.ThreadPoolExecutor() as executor:
            for entry in r:
                futures.append(executor.submit(_perform_read, self.endpoint, entry, self.session, self.df_facade))

            for f in futures:
                results.append(f.result())

        return results

    def read(self, r: FabrikReadRequest) -> FabrikReadResponse:
        return _perform_read(self.endpoint, r, self.session, self.df_facade)

    def write(self, r: FabrikWriteRequest) -> FabrikWriteResponse:
        pass
